/******************************************************************************
*                                                                             *
*    AuthenticManager.cpp                   Copyright(c) 2007-2010 itow,y.    *
*                                                                             *
******************************************************************************/

/*
  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


#include <windows.h>

#include "pntypes.h"
#include "pncom.h"

#include "Ant-RP.h"
#include "AuthenticManager.h"
#include "resource.h"


#define MAX_USERNAME_LENGTH	256
#define MAX_PASSWORD_LENGTH	256

struct PasswordInfo {
	char szUserName[MAX_USERNAME_LENGTH];
	char szPassword[MAX_PASSWORD_LENGTH];
};




AntAuthenticationManager::AntAuthenticationManager()
	: m_lRefCount(0)
{
}


AntAuthenticationManager::~AntAuthenticationManager()
{
}


static INT_PTR CALLBACK PasswordDlgProc(HWND hDlg,UINT uMsg,
												WPARAM wParam,LPARAM lParam)
{
	static PasswordInfo *pInfo;

	switch (uMsg) {
	case WM_INITDIALOG:
		pInfo=(PasswordInfo*)lParam;
		SetDlgItemTextA(hDlg,IDC_PASSWORD_USERNAME,pInfo->szUserName);
		SetDlgItemTextA(hDlg,IDC_PASSWORD_PASSWORD,pInfo->szPassword);
		return TRUE;

	case WM_COMMAND:
		switch (LOWORD(wParam)) {
		case IDOK:
			GetDlgItemTextA(hDlg,IDC_PASSWORD_USERNAME,pInfo->szUserName,
														MAX_USERNAME_LENGTH);
			GetDlgItemTextA(hDlg,IDC_PASSWORD_PASSWORD,pInfo->szPassword,
														MAX_PASSWORD_LENGTH);
		case IDCANCEL:
			EndDialog(hDlg,LOWORD(wParam));
			return TRUE;
		}
		break;
	}
	return FALSE;
}


STDMETHODIMP AntAuthenticationManager::HandleAuthenticationRequest(
								IRMAAuthenticationManagerResponse* pResponse)
{
	static PasswordInfo Info;

	if (DialogBoxParam(GetResourceInstance(),MAKEINTRESOURCE(IDD_PASSWORD),
						GetMainWindow(),PasswordDlgProc,(LPARAM)&Info)!=IDOK)
#if 0
		return pResponse->AuthenticationRequestDone(PNR_NOT_AUTHORIZED,
																	NULL,NULL);
#endif
		return PNR_FAIL;
	return pResponse->AuthenticationRequestDone(PNR_OK,Info.szUserName,
															Info.szPassword);
}


STDMETHODIMP_(ULONG) AntAuthenticationManager::AddRef()
{
	return InterlockedIncrement(&m_lRefCount);
}


STDMETHODIMP_(ULONG) AntAuthenticationManager::Release()
{
	if (InterlockedDecrement(&m_lRefCount)>0)
		return m_lRefCount;
	delete this;
	return 0;
}


STDMETHODIMP AntAuthenticationManager::QueryInterface(REFIID iid,void **ppObj)
{
	if (IsEqualIID(iid,IID_IUnknown)) {
		AddRef();
		*ppObj=static_cast<IUnknown*>(this);
		return PNR_OK;
	} else if (IsEqualIID(iid,IID_IRMAAuthenticationManager)) {
		AddRef();
		*ppObj=static_cast<IRMAAuthenticationManager*>(this);
		return PNR_OK;
	}
	*ppObj=NULL;
	return PNR_NOINTERFACE;
}
