/******************************************************************************
*                                                                             *
*    ClientAdviseSink.cpp                   Copyright(c) 2007-2010 itow,y.    *
*                                                                             *
******************************************************************************/

/*
  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


#include <windows.h>
#include <tchar.h>

#include "pntypes.h"
#include "pncom.h"
#include "rmacomm.h"
#include "rmamon.h"
#include "rmacore.h"
#include "rmaclsnk.h"

#include "Ant-RP.h"
#include "ClientAdviseSink.h"
#include "AuthenticManager.h"




static void SendNotify(int Type)
{
	SendMessage(GetMainWindow(),ANT_MESSAGE_NOTIFY,Type,0);
}


AntClientAdviseSink::AntClientAdviseSink(IUnknown *pUnknown)
{
	m_lRefCount=0;
	m_pRegistry=NULL;
	m_pTACManager=new AntTACManager;
	m_pTACManager->AddRef();
	m_pUnknown=pUnknown;
	if (pUnknown!=NULL) {
		m_pUnknown->AddRef();
		if (m_pUnknown->QueryInterface(IID_IRMAPNRegistry,
												(void**)&m_pRegistry)!=PNR_OK)
			m_pRegistry=NULL;
		IRMAPlayer *pPlayer;

		if (m_pUnknown->QueryInterface(IID_IRMAPlayer,(void**)&pPlayer)==
																	PNR_OK) {
			pPlayer->AddAdviseSink(this);
			m_pTACManager->Init(pPlayer);
			pPlayer->Release();
		}
	}
}


AntClientAdviseSink::~AntClientAdviseSink()
{
	if (m_pRegistry!=NULL)
		m_pRegistry->Release();
	if (m_pUnknown!=NULL)
		m_pUnknown->Release();
	if (m_pTACManager!=NULL) {
		m_pTACManager->Terminate();
		m_pTACManager->Release();
	}
}


void AntClientAdviseSink::ShowMeTheStatistics(const char *pszRegistryKey)
{
	const int Statistics=8;

	for (int i=0;i<Statistics;i++) {
		char szRegistryValue[MAX_DISPLAY_NAME];
		INT32 lValue=0;

		switch (i) {
		case 0:
			wsprintfA(szRegistryValue,"%s.Normal",pszRegistryKey);
			break;
		case 1:
			wsprintfA(szRegistryValue,"%s.Recovered",pszRegistryKey);
			break;
		case 2:
			wsprintfA(szRegistryValue,"%s.Received",pszRegistryKey);
	 		break;
		case 3:
			wsprintfA(szRegistryValue,"%s.Lost",pszRegistryKey);
			break;
		case 4:
			wsprintfA(szRegistryValue,"%s.Late",pszRegistryKey);
			break;
		case 5:
			wsprintfA(szRegistryValue,"%s.ClipBandwidth",pszRegistryKey);
			break;
		case 6:
			wsprintfA(szRegistryValue,"%s.AverageBandwidth",pszRegistryKey);
			break;
		case 7:
			wsprintfA(szRegistryValue,"%s.CurrentBandwidth",pszRegistryKey);
			break;
		}
		m_pRegistry->GetIntByName(szRegistryValue,lValue);
		//fprintf(stdout,"%s = %ld\n",szRegistryValue,lValue);
	}
}


STDMETHODIMP AntClientAdviseSink::OnPosLength(UINT32 ulPosition,
															UINT32 ulLength)
{
	SendMessage(GetMainWindow(),ANT_MESSAGE_POSLENGTH,ulPosition,ulLength);
	return PNR_OK;
}


STDMETHODIMP AntClientAdviseSink::OnPresentationOpened()
{
	SendNotify(NOTIFY_OPEN);
	return PNR_OK;
}


STDMETHODIMP AntClientAdviseSink::OnPresentationClosed()
{
	SendNotify(NOTIFY_CLOSE);
	return PNR_OK;
}


STDMETHODIMP AntClientAdviseSink::OnStatisticsChanged()
{
#if 0
	UINT32  unPlayerIndex=0;
	UINT32  unSourceIndex=0;
	UINT32  unStreamIndex=0;
	//INT32   nRegistryValue=0;
	static const char *pszRegistryPrefix="Statistics";
	char szRegistryName[MAX_DISPLAY_NAME];

	if (m_pRegistry!=NULL)  {
		wsprintfA(szRegistryName,"%s.Player%ld",pszRegistryPrefix,unPlayerIndex);
		while (m_pRegistry->GetTypeByName(szRegistryName)==PT_COMPOSITE) {
			ShowMeTheStatistics(szRegistryName);
			wsprintfA(szRegistryName,"%s.Source%ld",szRegistryName,unSourceIndex);
			while (m_pRegistry->GetTypeByName(szRegistryName)==PT_COMPOSITE) {
				ShowMeTheStatistics(szRegistryName);
				wsprintfA(szRegistryName,"%s.Stream%ld",
												szRegistryName,unStreamIndex);
				while (m_pRegistry->GetTypeByName(szRegistryName)==
																PT_COMPOSITE) {
					ShowMeTheStatistics(szRegistryName);
					unStreamIndex++;
					wsprintfA(szRegistryName,"%s.Player%ld.Source%ld.Stream%ld",
								pszRegistryPrefix,unPlayerIndex,unSourceIndex,
																unStreamIndex);
				}
				unSourceIndex++;
				wsprintfA(szRegistryName,"%s.Player%ld.Source%ld",
								pszRegistryPrefix,unPlayerIndex,unSourceIndex);
			}
			unPlayerIndex++;
			wsprintfA(szRegistryName,"%s.Player%ld",
											pszRegistryPrefix,unPlayerIndex);
		}
	}
#endif
	return PNR_OK;
}


STDMETHODIMP AntClientAdviseSink::OnPreSeek(UINT32 /*ulOldTime*/,UINT32 /*ulNewTime*/)
{
	//fprintf(stdout, "OnPreSeek(%ld, %ld)\n", ulOldTime, ulNewTime);
	return PNR_OK;
}


STDMETHODIMP AntClientAdviseSink::OnPostSeek(UINT32 /*ulOldTime*/,UINT32 /*ulNewTime*/)
{
	//fprintf(stdout, "OnPostSeek(%ld, %ld)\n", ulOldTime, ulNewTime);
	return PNR_OK;
}


STDMETHODIMP AntClientAdviseSink::OnStop()
{
	SendNotify(NOTIFY_STOP);
	return PNR_OK;
}


STDMETHODIMP AntClientAdviseSink::OnPause(UINT32 /*ulTime*/)
{
	//fprintf(stdout, "OnPause(%ld)\n",ulTime);
	SendNotify(NOTIFY_PAUSE);
	return PNR_OK;
}


STDMETHODIMP AntClientAdviseSink::OnBegin(UINT32 /*ulTime*/)
{
	m_pTACManager->Update();
	SendNotify(NOTIFY_PLAY);
#if 0
	if (m_pTACManager->GetTitle())
		printf("Title: %s\n",m_pTACManager->GetTitle());
	if (m_pTACManager->GetAuthor())
		printf("Author: %s\n",m_pTACManager->GetAuthor());
	if (m_pTACManager->GetCopyright())
		printf("Copyright: %s\n",m_pTACManager->GetCopyright());
	if (m_pTACManager->GetAbstract())
		printf("Abstract: %s\n",m_pTACManager->GetAbstract());
	if (m_pTACManager->GetKeywords())
		printf("Keywords: %s\n",m_pTACManager->GetKeywords());
#endif
	return PNR_OK;
}


STDMETHODIMP AntClientAdviseSink::OnBuffering(UINT32 /*ulFlags*/,
													UINT16 unPercentComplete)
{
	SendMessage(GetMainWindow(),ANT_MESSAGE_BUFFERING,0,unPercentComplete);
	return PNR_OK;
}


STDMETHODIMP AntClientAdviseSink::OnContacting(const char *pHostName)
{
	SendMessage(GetMainWindow(),ANT_MESSAGE_CONTACTING,0,(LPARAM)pHostName);
	return PNR_OK;
}


STDMETHODIMP_(ULONG) AntClientAdviseSink::AddRef()
{
	return InterlockedIncrement(&m_lRefCount);
}


STDMETHODIMP_(ULONG) AntClientAdviseSink::Release()
{
	if (InterlockedDecrement(&m_lRefCount)>0)
		return m_lRefCount;
	delete this;
	return 0;
}


STDMETHODIMP AntClientAdviseSink::QueryInterface(REFIID riid,void **ppObj)
{
	if (IsEqualIID(riid,IID_IUnknown)) {
		AddRef();
		*ppObj=static_cast<IUnknown*>(this);
		return PNR_OK;
	} else if (IsEqualIID(riid,IID_IRMAClientAdviseSink)) {
		AddRef();
		*ppObj=static_cast<IRMAClientAdviseSink*>(this);
		return PNR_OK;
	}
	*ppObj=NULL;
	return PNR_NOINTERFACE;
}
