/******************************************************************************
*                                                                             *
*    ErrorSink.cpp                          Copyright(c) 2007-2010 itow,y.    *
*                                                                             *
******************************************************************************/

/*
  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


#include <windows.h>
#include <tchar.h>

#include "pntypes.h"
#include "pncom.h"
#include "rmacomm.h"
#include "rmaerror.h"
#include "rmapckts.h"

#include "Ant-RP.h"
#include "ErrorSink.h"
#include "resource.h"


struct ErrorInfo {
	AntErrorSink *pErrorSink;
	UINT8 Severity;
	ULONG32 RMACode;
	ULONG32 UserCode;
	const char *pUserString;
	const char *pMoreInfoURL;
};




AntErrorSink::AntErrorSink(IUnknown *pUnknown)
	: m_lRefCount(0)
{
	if (pUnknown!=NULL) {
		m_pUnknown = pUnknown;
		m_pUnknown->AddRef();
	} else
		m_pUnknown=NULL;
}


AntErrorSink::~AntErrorSink()
{
	if (m_pUnknown!=NULL)
		m_pUnknown->Release();
}


IRMABuffer *AntErrorSink::GetErrorText(const ULONG32 RMACode)
{
	IRMAErrorMessages *pErrorMessages;
	IRMABuffer *pBuffer=NULL;

	if (m_pUnknown!=NULL && SUCCEEDED(m_pUnknown->QueryInterface(
							IID_IRMAErrorMessages,(void**)&pErrorMessages))) {
		pBuffer=pErrorMessages->GetErrorText(RMACode);
		pErrorMessages->Release();
	}
	return pBuffer;
}


INT_PTR CALLBACK AntErrorSink::ErrorDlgProc(HWND hDlg,UINT uMsg,
												WPARAM wParam,LPARAM lParam)
{
	switch (uMsg) {
	case WM_INITDIALOG:
		{
			ErrorInfo *pInfo=(ErrorInfo*)lParam;
			HINSTANCE hinst=GetResourceInstance();
			TCHAR szText[256];
			IRMABuffer *pBuffer=pInfo->pErrorSink->GetErrorText(pInfo->RMACode);
			int Length;
			char szUserCode[16];

			LoadString(hinst,IDS_ERROR_SEVERITY_FIRST+pInfo->Severity,
													szText,lengthof(szText));
			SetDlgItemText(hDlg,IDC_ERROR_SEVERITY,szText);
			wsprintf(szText,TEXT("0x%x"),pInfo->RMACode);
			SetDlgItemText(hDlg,IDC_ERROR_CODE,szText);
			Length=0;
			if (pInfo->pUserString!=NULL && pInfo->pUserString[0]!='\0') {
				wsprintfA(szUserCode,"(%d)\r\n",pInfo->UserCode);
				Length+=lstrlenA(pInfo->pUserString)+lstrlenA(szUserCode);
			}
			if (pInfo->pMoreInfoURL!=NULL && pInfo->pMoreInfoURL[0]!='\0')
				Length+=lstrlenA(pInfo->pMoreInfoURL)+2;
			if (pBuffer!=NULL)
				Length+=lstrlenA((char*)pBuffer->GetBuffer())+2;
			if (Length>0) {
				char *pszMessage=new char[Length+1];

				pszMessage[0]='\0';
				if (pInfo->pUserString!=NULL && pInfo->pUserString[0]!='\0') {
					lstrcatA(pszMessage,pInfo->pUserString);
					lstrcatA(pszMessage,szUserCode);
				}
				if (pInfo->pMoreInfoURL!=NULL && pInfo->pMoreInfoURL[0]!='\0') {
					lstrcatA(pszMessage,pInfo->pMoreInfoURL);
					lstrcatA(pszMessage,"\r\n");
				}
				if (pBuffer!=NULL) {
					lstrcatA(pszMessage,(char*)pBuffer->GetBuffer());
					lstrcatA(pszMessage,"\r\n");
				}
				SetDlgItemTextA(hDlg,IDC_ERROR_MESSAGE,pszMessage);
				delete [] pszMessage;
			}
			if (pBuffer!=NULL)
				pBuffer->Release();
		}
		return TRUE;

	case WM_COMMAND:
		switch (LOWORD(wParam)) {
		case IDOK:
		case IDCANCEL:
			EndDialog(hDlg,LOWORD(wParam));
			return TRUE;
		}
	}
	return FALSE;
}


STDMETHODIMP AntErrorSink::ErrorOccurred(const UINT8 Severity,
		const ULONG32 RMACode,const ULONG32 UserCode,const char *pUserString,
													const char *pMoreInfoURL)
{
	ErrorInfo Info;

	Info.pErrorSink=this;
	Info.Severity=Severity;
	Info.RMACode=RMACode;
	Info.UserCode=UserCode;
	Info.pUserString=pUserString;
	Info.pMoreInfoURL=pMoreInfoURL;
	DialogBoxParam(GetResourceInstance(),MAKEINTRESOURCE(IDD_ERROR),
								GetMainWindow(),ErrorDlgProc,(LPARAM)&Info);
	return PNR_OK;
}


STDMETHODIMP_(ULONG) AntErrorSink::AddRef()
{
	return InterlockedIncrement(&m_lRefCount);
}


STDMETHODIMP_(ULONG) AntErrorSink::Release()
{
	if (InterlockedDecrement(&m_lRefCount)>0)
		return m_lRefCount;
	delete this;
	return 0;
}


STDMETHODIMP AntErrorSink::QueryInterface(REFIID iid, void **ppObj)
{
	if (IsEqualIID(iid,IID_IUnknown)) {
		AddRef();
		*ppObj=static_cast<IUnknown*>(this);
		return PNR_OK;
	} else if (IsEqualIID(iid,IID_IRMAErrorSink)) {
		AddRef();
		*ppObj=static_cast<IRMAErrorSink*>(this);
		return PNR_OK;
	}
	*ppObj=NULL;
	return PNR_NOINTERFACE;
}
