/******************************************************************************
*                                                                             *
*    Settings.cpp                           Copyright(c) 2007-2010 itow,y.    *
*                                                                             *
******************************************************************************/

/*
  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


#include <windows.h>
#include <shlwapi.h>
#include <tchar.h>
#include "Settings.h"




AntSettings::AntSettings()
	: m_OpenFlags(0)
{
}


AntSettings::~AntSettings()
{
	Close();
}




AntRegSettings::AntRegSettings()
	: m_hkey(NULL)
{
}


AntRegSettings::~AntRegSettings()
{
}


bool AntRegSettings::Open(HKEY hkey,LPCTSTR pszSubKey,unsigned int Flags)
{
	bool fRet;

	if ((Flags&OPEN_WRITE)==0) {
		fRet=RegOpenKeyEx(hkey,pszSubKey,0,KEY_READ,&m_hkey)==ERROR_SUCCESS;
	} else {
		DWORD dwResult;

		fRet=RegCreateKeyEx(hkey,pszSubKey,0,TEXT(""),REG_OPTION_NON_VOLATILE,
						KEY_ALL_ACCESS,NULL,&m_hkey,&dwResult)==ERROR_SUCCESS;
	}
	m_OpenFlags=Flags;
	return fRet;
}


void AntRegSettings::Close()
{
	if (m_hkey!=NULL) {
		RegCloseKey(m_hkey);
		m_hkey=NULL;
	}
}


bool AntRegSettings::WriteInt(LPCTSTR pszName,int Value)
{
	LONG lData=Value;

	return RegSetValueEx(m_hkey,pszName,0,REG_DWORD,(BYTE*)&lData,
												sizeof(DWORD))==ERROR_SUCCESS;
}


bool AntRegSettings::ReadInt(LPCTSTR pszName,int *pValue)
{
	DWORD dwType,dwBytes=sizeof(DWORD);
	LONG lData;

	if (RegQueryValueEx(m_hkey,pszName,NULL,&dwType,(BYTE*)&lData,&dwBytes)!=
																ERROR_SUCCESS)
		return false;
	*pValue=lData;
	return true;
}


bool AntRegSettings::WriteString(LPCTSTR pszName,LPCTSTR pszString)
{
	return RegSetValueEx(m_hkey,pszName,0,REG_SZ,(BYTE*)pszString,
						(lstrlen(pszString)+1)*sizeof(TCHAR))==ERROR_SUCCESS;
}


bool AntRegSettings::ReadString(LPCTSTR pszName,LPTSTR pszString,int MaxLength)
{
	DWORD dwType=REG_SZ,dwDataBytes=MaxLength*sizeof(TCHAR);

	return RegQueryValueEx(m_hkey,pszName,NULL,&dwType,(BYTE*)pszString,
												&dwDataBytes)==ERROR_SUCCESS;
}




AntIniSettings::AntIniSettings()
{
}


AntIniSettings::~AntIniSettings()
{
}


bool AntIniSettings::Open(LPCTSTR pszFileName,LPCTSTR pszSection,
															unsigned int Flags)
{
	lstrcpy(m_szFileName,pszFileName);
	lstrcpy(m_szSection,pszSection);
	m_OpenFlags=Flags;
	return true;
}


void AntIniSettings::Close()
{
	if ((m_OpenFlags&OPEN_WRITE)!=0)
		WritePrivateProfileString(NULL,NULL,NULL,m_szFileName);	// Flush
	m_OpenFlags=0;
}


bool AntIniSettings::ReadInt(LPCTSTR pszValueName,int *pValue)
{
	TCHAR szValue[16];

	if (!ReadString(pszValueName,szValue,16) || szValue[0]=='\0')
		return false;
	*pValue=StrToInt(szValue);
	return true;
}


bool AntIniSettings::WriteInt(LPCTSTR pszValueName,int Value)
{
	TCHAR szValue[16];

	wsprintf(szValue,TEXT("%d"),Value);
	return WriteString(pszValueName,szValue);
}


bool AntIniSettings::ReadString(LPCTSTR pszValueName,LPTSTR pszString,
																int MaxLength)
{
	TCHAR cBack[2];

	cBack[0]=pszString[0];
	if (MaxLength>1)
		cBack[1]=pszString[1];
	GetPrivateProfileString(m_szSection,pszValueName,TEXT("\x1A"),
											pszString,MaxLength,m_szFileName);
	if (pszString[0]=='\x1A') {
		pszString[0]=cBack[0];
		if (MaxLength>1)
			pszString[1]=cBack[1];
		return false;
	}
	return true;
}


bool AntIniSettings::WriteString(LPCTSTR pszValueName,LPCTSTR pszValue)
{
	return WritePrivateProfileString(m_szSection,pszValueName,pszValue,
														m_szFileName)!=FALSE;
}
